/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface. The Ed-Fi ODS / API supports both transactional and bulk modes of operation.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using EdFi.OdsApi.Sdk.Client;
using EdFi.OdsApi.Sdk.Models.Resources;

namespace EdFi.OdsApi.Sdk.Api.Resources
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public interface ICommunityProviderLicensesApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// Deletes an existing resource using the resource identifier.
        /// </summary>
        /// <remarks>
        /// The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns></returns>
        void DeleteCommunityProviderLicenseById (string id, string ifMatch = null);

        /// <summary>
        /// Deletes an existing resource using the resource identifier.
        /// </summary>
        /// <remarks>
        /// The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<Object> DeleteCommunityProviderLicenseByIdWithHttpInfo (string id, string ifMatch = null);
        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>List&lt;EdFiCommunityProviderLicense&gt;</returns>
        List<EdFiCommunityProviderLicense> GetCommunityProviderLicenses (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null);

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>ApiResponse of List&lt;EdFiCommunityProviderLicense&gt;</returns>
        ApiResponse<List<EdFiCommunityProviderLicense>> GetCommunityProviderLicensesWithHttpInfo (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null);
        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation retrieves a resource by the specified resource identifier.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>EdFiCommunityProviderLicense</returns>
        EdFiCommunityProviderLicense GetCommunityProviderLicensesById (string id, string ifNoneMatch = null);

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation retrieves a resource by the specified resource identifier.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>ApiResponse of EdFiCommunityProviderLicense</returns>
        ApiResponse<EdFiCommunityProviderLicense> GetCommunityProviderLicensesByIdWithHttpInfo (string id, string ifNoneMatch = null);
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource.
        /// </summary>
        /// <remarks>
        /// The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns></returns>
        void PostCommunityProviderLicense (EdFiCommunityProviderLicense communityProviderLicense);

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource.
        /// </summary>
        /// <remarks>
        /// The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<Object> PostCommunityProviderLicenseWithHttpInfo (EdFiCommunityProviderLicense communityProviderLicense);
        /// <summary>
        /// Updates or creates a resource based on the resource identifier.
        /// </summary>
        /// <remarks>
        /// The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns></returns>
        void PutCommunityProviderLicense (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null);

        /// <summary>
        /// Updates or creates a resource based on the resource identifier.
        /// </summary>
        /// <remarks>
        /// The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<Object> PutCommunityProviderLicenseWithHttpInfo (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null);
        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// Deletes an existing resource using the resource identifier.
        /// </summary>
        /// <remarks>
        /// The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task DeleteCommunityProviderLicenseByIdAsync (string id, string ifMatch = null);

        /// <summary>
        /// Deletes an existing resource using the resource identifier.
        /// </summary>
        /// <remarks>
        /// The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<Object>> DeleteCommunityProviderLicenseByIdAsyncWithHttpInfo (string id, string ifMatch = null);
        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>Task of List&lt;EdFiCommunityProviderLicense&gt;</returns>
        System.Threading.Tasks.Task<List<EdFiCommunityProviderLicense>> GetCommunityProviderLicensesAsync (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null);

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>Task of ApiResponse (List&lt;EdFiCommunityProviderLicense&gt;)</returns>
        System.Threading.Tasks.Task<ApiResponse<List<EdFiCommunityProviderLicense>>> GetCommunityProviderLicensesAsyncWithHttpInfo (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null);
        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation retrieves a resource by the specified resource identifier.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>Task of EdFiCommunityProviderLicense</returns>
        System.Threading.Tasks.Task<EdFiCommunityProviderLicense> GetCommunityProviderLicensesByIdAsync (string id, string ifNoneMatch = null);

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern).
        /// </summary>
        /// <remarks>
        /// This GET operation retrieves a resource by the specified resource identifier.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>Task of ApiResponse (EdFiCommunityProviderLicense)</returns>
        System.Threading.Tasks.Task<ApiResponse<EdFiCommunityProviderLicense>> GetCommunityProviderLicensesByIdAsyncWithHttpInfo (string id, string ifNoneMatch = null);
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource.
        /// </summary>
        /// <remarks>
        /// The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task PostCommunityProviderLicenseAsync (EdFiCommunityProviderLicense communityProviderLicense);

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource.
        /// </summary>
        /// <remarks>
        /// The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<Object>> PostCommunityProviderLicenseAsyncWithHttpInfo (EdFiCommunityProviderLicense communityProviderLicense);
        /// <summary>
        /// Updates or creates a resource based on the resource identifier.
        /// </summary>
        /// <remarks>
        /// The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task PutCommunityProviderLicenseAsync (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null);

        /// <summary>
        /// Updates or creates a resource based on the resource identifier.
        /// </summary>
        /// <remarks>
        /// The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </remarks>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<Object>> PutCommunityProviderLicenseAsyncWithHttpInfo (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
    public partial class CommunityProviderLicensesApi : ICommunityProviderLicensesApi
    {
        private EdFi.OdsApi.Sdk.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="CommunityProviderLicensesApi"/> class.
        /// </summary>
        /// <returns></returns>
        public CommunityProviderLicensesApi(String basePath)
        {
            this.Configuration = new Configuration { BasePath = basePath };

            ExceptionFactory = EdFi.OdsApi.Sdk.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommunityProviderLicensesApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public CommunityProviderLicensesApi(Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = EdFi.OdsApi.Sdk.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public EdFi.OdsApi.Sdk.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public IDictionary<String, String> DefaultHeader()
        {
            return new ReadOnlyDictionary<string, string>(this.Configuration.DefaultHeader);
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns></returns>
        public void DeleteCommunityProviderLicenseById (string id, string ifMatch = null)
        {
             DeleteCommunityProviderLicenseByIdWithHttpInfo(id, ifMatch);
        }

        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<Object> DeleteCommunityProviderLicenseByIdWithHttpInfo (string id, string ifMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->DeleteCommunityProviderLicenseById");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifMatch != null) localVarHeaderParams.Add("If-Match", Configuration.ApiClient.ParameterToString(ifMatch)); // header parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.DELETE, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("DeleteCommunityProviderLicenseById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task DeleteCommunityProviderLicenseByIdAsync (string id, string ifMatch = null)
        {
             await DeleteCommunityProviderLicenseByIdAsyncWithHttpInfo(id, ifMatch);

        }

        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier. If the resource doesn&#39;t exist, an error will result (the resource will not be found).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer. (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<Object>> DeleteCommunityProviderLicenseByIdAsyncWithHttpInfo (string id, string ifMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->DeleteCommunityProviderLicenseById");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifMatch != null) localVarHeaderParams.Add("If-Match", Configuration.ApiClient.ParameterToString(ifMatch)); // header parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.DELETE, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("DeleteCommunityProviderLicenseById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern). This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>List&lt;EdFiCommunityProviderLicense&gt;</returns>
        public List<EdFiCommunityProviderLicense> GetCommunityProviderLicenses (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null)
        {
             ApiResponse<List<EdFiCommunityProviderLicense>> localVarResponse = GetCommunityProviderLicensesWithHttpInfo(offset, limit, totalCount, licenseIdentifier, licensingOrganization, communityProviderId, licenseStatusDescriptor, licenseTypeDescriptor, authorizedFacilityCapacity, id, licenseEffectiveDate, licenseExpirationDate, licenseIssueDate, oldestAgeAuthorizedToServe, youngestAgeAuthorizedToServe);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern). This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>ApiResponse of List&lt;EdFiCommunityProviderLicense&gt;</returns>
        public ApiResponse< List<EdFiCommunityProviderLicense> > GetCommunityProviderLicensesWithHttpInfo (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null)
        {

            var localVarPath = "/ed-fi/communityProviderLicenses";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (offset != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "offset", offset)); // query parameter
            if (limit != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "limit", limit)); // query parameter
            if (totalCount != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "totalCount", totalCount)); // query parameter
            if (licenseIdentifier != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseIdentifier", licenseIdentifier)); // query parameter
            if (licensingOrganization != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licensingOrganization", licensingOrganization)); // query parameter
            if (communityProviderId != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "communityProviderId", communityProviderId)); // query parameter
            if (licenseStatusDescriptor != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseStatusDescriptor", licenseStatusDescriptor)); // query parameter
            if (licenseTypeDescriptor != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseTypeDescriptor", licenseTypeDescriptor)); // query parameter
            if (authorizedFacilityCapacity != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "authorizedFacilityCapacity", authorizedFacilityCapacity)); // query parameter
            if (id != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "id", id)); // query parameter
            if (licenseEffectiveDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseEffectiveDate", licenseEffectiveDate)); // query parameter
            if (licenseExpirationDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseExpirationDate", licenseExpirationDate)); // query parameter
            if (licenseIssueDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseIssueDate", licenseIssueDate)); // query parameter
            if (oldestAgeAuthorizedToServe != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "oldestAgeAuthorizedToServe", oldestAgeAuthorizedToServe)); // query parameter
            if (youngestAgeAuthorizedToServe != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "youngestAgeAuthorizedToServe", youngestAgeAuthorizedToServe)); // query parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetCommunityProviderLicenses", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<List<EdFiCommunityProviderLicense>>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (List<EdFiCommunityProviderLicense>) Configuration.ApiClient.Deserialize(localVarResponse, typeof(List<EdFiCommunityProviderLicense>)));
        }

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern). This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>Task of List&lt;EdFiCommunityProviderLicense&gt;</returns>
        public async System.Threading.Tasks.Task<List<EdFiCommunityProviderLicense>> GetCommunityProviderLicensesAsync (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null)
        {
             ApiResponse<List<EdFiCommunityProviderLicense>> localVarResponse = await GetCommunityProviderLicensesAsyncWithHttpInfo(offset, limit, totalCount, licenseIdentifier, licensingOrganization, communityProviderId, licenseStatusDescriptor, licenseTypeDescriptor, authorizedFacilityCapacity, id, licenseEffectiveDate, licenseExpirationDate, licenseIssueDate, oldestAgeAuthorizedToServe, youngestAgeAuthorizedToServe);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Retrieves specific resources using the resource&#39;s property values (using the \&quot;Get\&quot; pattern). This GET operation provides access to resources using the \&quot;Get\&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="offset">Indicates how many items should be skipped before returning results. (optional, default to 0)</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results. (optional, default to 25)</param>
        /// <param name="totalCount">Indicates if the total number of items available should be returned in the &#39;Total-Count&#39; header of the response.  If set to false, &#39;Total-Count&#39; header will not be provided. (optional, default to false)</param>
        /// <param name="licenseIdentifier">The unique identifier issued by the licensing organization. (optional)</param>
        /// <param name="licensingOrganization">The organization issuing the license. (optional)</param>
        /// <param name="communityProviderId">The identifier assigned to a CommunityProvider. (optional)</param>
        /// <param name="licenseStatusDescriptor">An indication of the status of the license. (optional)</param>
        /// <param name="licenseTypeDescriptor">An indication of the category of the license. (optional)</param>
        /// <param name="authorizedFacilityCapacity">The maximum number that can be contained or accommodated which a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="id"> (optional)</param>
        /// <param name="licenseEffectiveDate">The month, day, and year on which a license is active or becomes effective. (optional)</param>
        /// <param name="licenseExpirationDate">The month, day, and year on which a license will expire. (optional)</param>
        /// <param name="licenseIssueDate">The month, day, and year on which an active license was issued. (optional)</param>
        /// <param name="oldestAgeAuthorizedToServe">The oldest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <param name="youngestAgeAuthorizedToServe">The youngest age of children a provider is authorized or licensed to serve. (optional)</param>
        /// <returns>Task of ApiResponse (List&lt;EdFiCommunityProviderLicense&gt;)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<List<EdFiCommunityProviderLicense>>> GetCommunityProviderLicensesAsyncWithHttpInfo (int? offset = null, int? limit = null, bool? totalCount = null, string licenseIdentifier = null, string licensingOrganization = null, int? communityProviderId = null, string licenseStatusDescriptor = null, string licenseTypeDescriptor = null, int? authorizedFacilityCapacity = null, string id = null, DateTime? licenseEffectiveDate = null, DateTime? licenseExpirationDate = null, DateTime? licenseIssueDate = null, int? oldestAgeAuthorizedToServe = null, int? youngestAgeAuthorizedToServe = null)
        {

            var localVarPath = "/ed-fi/communityProviderLicenses";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (offset != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "offset", offset)); // query parameter
            if (limit != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "limit", limit)); // query parameter
            if (totalCount != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "totalCount", totalCount)); // query parameter
            if (licenseIdentifier != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseIdentifier", licenseIdentifier)); // query parameter
            if (licensingOrganization != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licensingOrganization", licensingOrganization)); // query parameter
            if (communityProviderId != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "communityProviderId", communityProviderId)); // query parameter
            if (licenseStatusDescriptor != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseStatusDescriptor", licenseStatusDescriptor)); // query parameter
            if (licenseTypeDescriptor != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseTypeDescriptor", licenseTypeDescriptor)); // query parameter
            if (authorizedFacilityCapacity != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "authorizedFacilityCapacity", authorizedFacilityCapacity)); // query parameter
            if (id != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "id", id)); // query parameter
            if (licenseEffectiveDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseEffectiveDate", licenseEffectiveDate)); // query parameter
            if (licenseExpirationDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseExpirationDate", licenseExpirationDate)); // query parameter
            if (licenseIssueDate != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "licenseIssueDate", licenseIssueDate)); // query parameter
            if (oldestAgeAuthorizedToServe != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "oldestAgeAuthorizedToServe", oldestAgeAuthorizedToServe)); // query parameter
            if (youngestAgeAuthorizedToServe != null) localVarQueryParams.AddRange(Configuration.ApiClient.ParameterToKeyValuePairs("", "youngestAgeAuthorizedToServe", youngestAgeAuthorizedToServe)); // query parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetCommunityProviderLicenses", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<List<EdFiCommunityProviderLicense>>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (List<EdFiCommunityProviderLicense>) Configuration.ApiClient.Deserialize(localVarResponse, typeof(List<EdFiCommunityProviderLicense>)));
        }

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>EdFiCommunityProviderLicense</returns>
        public EdFiCommunityProviderLicense GetCommunityProviderLicensesById (string id, string ifNoneMatch = null)
        {
             ApiResponse<EdFiCommunityProviderLicense> localVarResponse = GetCommunityProviderLicensesByIdWithHttpInfo(id, ifNoneMatch);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>ApiResponse of EdFiCommunityProviderLicense</returns>
        public ApiResponse< EdFiCommunityProviderLicense > GetCommunityProviderLicensesByIdWithHttpInfo (string id, string ifNoneMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->GetCommunityProviderLicensesById");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifNoneMatch != null) localVarHeaderParams.Add("If-None-Match", Configuration.ApiClient.ParameterToString(ifNoneMatch)); // header parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetCommunityProviderLicensesById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<EdFiCommunityProviderLicense>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (EdFiCommunityProviderLicense) Configuration.ApiClient.Deserialize(localVarResponse, typeof(EdFiCommunityProviderLicense)));
        }

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>Task of EdFiCommunityProviderLicense</returns>
        public async System.Threading.Tasks.Task<EdFiCommunityProviderLicense> GetCommunityProviderLicensesByIdAsync (string id, string ifNoneMatch = null)
        {
             ApiResponse<EdFiCommunityProviderLicense> localVarResponse = await GetCommunityProviderLicensesByIdAsyncWithHttpInfo(id, ifNoneMatch);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Retrieves a specific resource using the resource&#39;s identifier (using the \&quot;Get By Id\&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="ifNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource. (optional)</param>
        /// <returns>Task of ApiResponse (EdFiCommunityProviderLicense)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<EdFiCommunityProviderLicense>> GetCommunityProviderLicensesByIdAsyncWithHttpInfo (string id, string ifNoneMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->GetCommunityProviderLicensesById");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifNoneMatch != null) localVarHeaderParams.Add("If-None-Match", Configuration.ApiClient.ParameterToString(ifNoneMatch)); // header parameter

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetCommunityProviderLicensesById", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<EdFiCommunityProviderLicense>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                (EdFiCommunityProviderLicense) Configuration.ApiClient.Deserialize(localVarResponse, typeof(EdFiCommunityProviderLicense)));
        }

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns></returns>
        public void PostCommunityProviderLicense (EdFiCommunityProviderLicense communityProviderLicense)
        {
             PostCommunityProviderLicenseWithHttpInfo(communityProviderLicense);
        }

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<Object> PostCommunityProviderLicenseWithHttpInfo (EdFiCommunityProviderLicense communityProviderLicense)
        {
            // verify the required parameter 'communityProviderLicense' is set
            if (communityProviderLicense == null)
                throw new ApiException(400, "Missing required parameter 'communityProviderLicense' when calling CommunityProviderLicensesApi->PostCommunityProviderLicense");

            var localVarPath = "/ed-fi/communityProviderLicenses";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (communityProviderLicense != null && communityProviderLicense.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(communityProviderLicense); // http body (model) parameter
            }
            else
            {
                localVarPostBody = communityProviderLicense; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostCommunityProviderLicense", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task PostCommunityProviderLicenseAsync (EdFiCommunityProviderLicense communityProviderLicense)
        {
             await PostCommunityProviderLicenseAsyncWithHttpInfo(communityProviderLicense);

        }

        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an \&quot;upsert\&quot; operation (insert + update). Clients should NOT include the resource \&quot;id\&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by \&quot;id\&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<Object>> PostCommunityProviderLicenseAsyncWithHttpInfo (EdFiCommunityProviderLicense communityProviderLicense)
        {
            // verify the required parameter 'communityProviderLicense' is set
            if (communityProviderLicense == null)
                throw new ApiException(400, "Missing required parameter 'communityProviderLicense' when calling CommunityProviderLicensesApi->PostCommunityProviderLicense");

            var localVarPath = "/ed-fi/communityProviderLicenses";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (communityProviderLicense != null && communityProviderLicense.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(communityProviderLicense); // http body (model) parameter
            }
            else
            {
                localVarPostBody = communityProviderLicense; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.POST, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PostCommunityProviderLicense", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns></returns>
        public void PutCommunityProviderLicense (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null)
        {
             PutCommunityProviderLicenseWithHttpInfo(id, communityProviderLicense, ifMatch);
        }

        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<Object> PutCommunityProviderLicenseWithHttpInfo (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->PutCommunityProviderLicense");
            // verify the required parameter 'communityProviderLicense' is set
            if (communityProviderLicense == null)
                throw new ApiException(400, "Missing required parameter 'communityProviderLicense' when calling CommunityProviderLicensesApi->PutCommunityProviderLicense");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifMatch != null) localVarHeaderParams.Add("If-Match", Configuration.ApiClient.ParameterToString(ifMatch)); // header parameter
            if (communityProviderLicense != null && communityProviderLicense.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(communityProviderLicense); // http body (model) parameter
            }
            else
            {
                localVarPostBody = communityProviderLicense; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) Configuration.ApiClient.CallApi(localVarPath,
                Method.PUT, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PutCommunityProviderLicense", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task PutCommunityProviderLicenseAsync (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null)
        {
             await PutCommunityProviderLicenseAsyncWithHttpInfo(id, communityProviderLicense, ifMatch);

        }

        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier. If the resource doesn&#39;t exist, the resource will be created using that identifier. Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource \&quot;id\&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <exception cref="EdFi.OdsApi.Sdk.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="id">A resource identifier that uniquely identifies the resource.</param>
        /// <param name="communityProviderLicense">The JSON representation of the \&quot;communityProviderLicense\&quot; resource to be created or updated.</param>
        /// <param name="ifMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer. (optional)</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<Object>> PutCommunityProviderLicenseAsyncWithHttpInfo (string id, EdFiCommunityProviderLicense communityProviderLicense, string ifMatch = null)
        {
            // verify the required parameter 'id' is set
            if (id == null)
                throw new ApiException(400, "Missing required parameter 'id' when calling CommunityProviderLicensesApi->PutCommunityProviderLicense");
            // verify the required parameter 'communityProviderLicense' is set
            if (communityProviderLicense == null)
                throw new ApiException(400, "Missing required parameter 'communityProviderLicense' when calling CommunityProviderLicensesApi->PutCommunityProviderLicense");

            var localVarPath = "/ed-fi/communityProviderLicenses/{id}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (id != null) localVarPathParams.Add("id", Configuration.ApiClient.ParameterToString(id)); // path parameter
            if (ifMatch != null) localVarHeaderParams.Add("If-Match", Configuration.ApiClient.ParameterToString(ifMatch)); // header parameter
            if (communityProviderLicense != null && communityProviderLicense.GetType() != typeof(byte[]))
            {
                localVarPostBody = Configuration.ApiClient.Serialize(communityProviderLicense); // http body (model) parameter
            }
            else
            {
                localVarPostBody = communityProviderLicense; // byte array
            }

            // authentication (oauth2_client_credentials) required
            // oauth required
            if (!String.IsNullOrEmpty(Configuration.AccessToken))
            {
                localVarHeaderParams["Authorization"] = "Bearer " + Configuration.AccessToken;
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.PUT, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PutCommunityProviderLicense", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => x.Value.ToString()),
                null);
        }

    }
}
