/* 
 * Ed-Fi Operational Data Store API
 *
 * The Ed-Fi ODS / API enables applications to read and write education data stored in an Ed-Fi ODS through a secure REST interface. The Ed-Fi ODS / API supports both transactional and bulk modes of operation.  ***  > *Note: Consumers of ODS / API information should sanitize all data for display and storage. The ODS / API provides reasonable safeguards against cross-site scripting attacks and other malicious content, but the platform does not and cannot guarantee that the data it contains is free of all potentially harmful content.*  *** 
 *
 * OpenAPI spec version: 3.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = EdFi.OdsApi.Sdk.Client.SwaggerDateConverter;

namespace EdFi.OdsApi.Sdk.Models.Resources
{
    /// <summary>
    /// EdFiDisciplineIncidentBehavior
    /// </summary>
    [DataContract]
    public partial class EdFiDisciplineIncidentBehavior :  IEquatable<EdFiDisciplineIncidentBehavior>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncidentBehavior" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected EdFiDisciplineIncidentBehavior() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="EdFiDisciplineIncidentBehavior" /> class.
        /// </summary>
        /// <param name="BehaviorDescriptor">Describes behavior by category and provides a detailed description. (required).</param>
        /// <param name="BehaviorDetailedDescription">Specifies a more granular level of detail of a behavior involved in the incident..</param>
        public EdFiDisciplineIncidentBehavior(string BehaviorDescriptor = default(string), string BehaviorDetailedDescription = default(string))
        {
            // to ensure "BehaviorDescriptor" is required (not null)
            if (BehaviorDescriptor == null)
            {
                throw new InvalidDataException("BehaviorDescriptor is a required property for EdFiDisciplineIncidentBehavior and cannot be null");
            }
            else
            {
                this.BehaviorDescriptor = BehaviorDescriptor;
            }
            this.BehaviorDetailedDescription = BehaviorDetailedDescription;
        }
        
        /// <summary>
        /// Describes behavior by category and provides a detailed description.
        /// </summary>
        /// <value>Describes behavior by category and provides a detailed description.</value>
        [DataMember(Name="behaviorDescriptor", EmitDefaultValue=false)]
        public string BehaviorDescriptor { get; set; }

        /// <summary>
        /// Specifies a more granular level of detail of a behavior involved in the incident.
        /// </summary>
        /// <value>Specifies a more granular level of detail of a behavior involved in the incident.</value>
        [DataMember(Name="behaviorDetailedDescription", EmitDefaultValue=false)]
        public string BehaviorDetailedDescription { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class EdFiDisciplineIncidentBehavior {\n");
            sb.Append("  BehaviorDescriptor: ").Append(BehaviorDescriptor).Append("\n");
            sb.Append("  BehaviorDetailedDescription: ").Append(BehaviorDetailedDescription).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as EdFiDisciplineIncidentBehavior);
        }

        /// <summary>
        /// Returns true if EdFiDisciplineIncidentBehavior instances are equal
        /// </summary>
        /// <param name="input">Instance of EdFiDisciplineIncidentBehavior to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(EdFiDisciplineIncidentBehavior input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.BehaviorDescriptor == input.BehaviorDescriptor ||
                    (this.BehaviorDescriptor != null &&
                    this.BehaviorDescriptor.Equals(input.BehaviorDescriptor))
                ) && 
                (
                    this.BehaviorDetailedDescription == input.BehaviorDetailedDescription ||
                    (this.BehaviorDetailedDescription != null &&
                    this.BehaviorDetailedDescription.Equals(input.BehaviorDetailedDescription))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.BehaviorDescriptor != null)
                    hashCode = hashCode * 59 + this.BehaviorDescriptor.GetHashCode();
                if (this.BehaviorDetailedDescription != null)
                    hashCode = hashCode * 59 + this.BehaviorDetailedDescription.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // BehaviorDescriptor (string) maxLength
            if(this.BehaviorDescriptor != null && this.BehaviorDescriptor.Length > 306)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BehaviorDescriptor, length must be less than 306.", new [] { "BehaviorDescriptor" });
            }

            // BehaviorDetailedDescription (string) maxLength
            if(this.BehaviorDetailedDescription != null && this.BehaviorDetailedDescription.Length > 1024)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for BehaviorDetailedDescription, length must be less than 1024.", new [] { "BehaviorDetailedDescription" });
            }

            yield break;
        }
    }

}
